"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/gallery.py $
$Id: gallery.py 23659 2004-03-08 22:44:18Z dbinger $
"""

from dulcinea.base import DulcineaPersistent
from dulcinea.attachable import Attachable

class Gallery (DulcineaPersistent, Attachable):
    """
    Instance attributes: none
    """

    def get_allowed_mime_types(self, user=None):
        return ['image/jpeg',
                'image/gif',
                'image/png']

