#!/www/python/bin/python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/bin/unknown.py $
$Id: unknown.py 23142 2003-11-11 21:18:02Z dbinger $
"""

from dulcinea.code_util import report_unknown_names, main

main(report_unknown_names)
